/* Copyright (c) 2008, "Jonas Finnemann Jensen" <jopsen@gmail.com>,
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following  *conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation 
 *      and/or other materials provided with the distribution.
 *    * Neither the name of the <ORGANIZATION> nor the names of its contributors
 *      may be used to endorse or promote products derived from this software without
 *      specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include "HSH.h"

/*Private helper functions*/
uint32_t rotate(uint32_t in, uint32_t places);
inline void endian_swap(uint32_t* x);

/*Initialize a HSHContext*/
void HSHInit(HSHContext* context)
{
	context->state = 0x40490FDB; //pi in IEEE-754 format
	context->result = 0;
}

/*Feed a Context with raw input*/
void HSHRawInput(HSHContext* context, uint32_t* keyunit)
{
	uint8_t i;
	context->result ^= *keyunit;
	for(i=0; i<7; i++)
	{
		context->state = rotate(context->state, 11);
		context->result = rotate(context->result, 13);
		context->result ^= context->state;
		context->result = rotate(context->result, context->state);
		context->state = rotate(context->state, context->result);
	}
}

/*Feed a Context with a string*/
void HSHStrInput(HSHContext* context, char* key)
{
	uint32_t temp = strlen(key);
	uint8_t rest = temp%4;
	uint8_t i, loop = temp-rest;
	for(i=0; i < loop; i+=4)
	{
		memcpy(&temp, key+i, 4);
		endian_swap(&temp);
		HSHRawInput(context, &temp);
	}
	if(rest != 0)
	{
		temp = 0;
		memcpy(&temp, key+i,rest);
		endian_swap(&temp);
		HSHRawInput(context, &temp);
	}
}

/*Result the result from a HSHContext*/
void HSHResult(HSHContext* context, uint32_t* result)
{
	*result = context->result;
}

/*Reset a HSHContext, same as HSHInit()*/
void HSHReset(HSHContext* context)
{
	HSHInit(context);
}

/*Rotate bis left*/
uint32_t rotate(uint32_t in, uint32_t places)
{
	//One may want to optimize this with assembler
	uint32_t out;
	out = in << (places % (8*sizeof(uint32_t)));
	out |= in >> ((8*sizeof(uint32_t)) - places % (8*sizeof(uint32_t)));
	return out;
}

/*Swaps the endian*/
inline void endian_swap(uint32_t* x)
{
	*x =	(*x>>24) | 
		((*x<<8) & 0x00FF0000) |
		((*x>>8) & 0x0000FF00) |
		(*x<<24);
}

